' ****************************************************************************************
'
'   GENERAL DEFINITIONS
'
' ****************************************************************************************
  $regfile = "2313def.dat"                                  'regfile must match the chip
  $crystal = 4000000                                        'crystal must match too
' ****************************************************************************************

  Const Preset = 0                                          ' T/C0 Preset Constant(256 - x)
  Const Offset = 10                                         ' Offset Constant
  Const Full = 215                                          ' Battery full Constant  (13.8 V)
  Const Deep = 145                                          ' Battery low Constant   (10.5 V)
  Const Empty = 118                                         ' Battery empty Constant ( 8.5 V)
  Const Dimmed_on = 5                                       ' Upper constant for dimmed brightness
  Const Dimmed_off = 3                                      ' Constant for dimmed brightness

' ****************************************************************************************

  Ddrb = &B00111100                                         ' Use pin 2..5 as outputs
  Ddrd = &B01111111                                         ' Use pin 0..6 as outputs

  Vmess Alias Portb.1                                       ' D1  PB1 AI1 Voltage input
  Vramp Alias Portb.0                                       ' D0  PB0 AI0 Ramp compare input
  Vswitch Alias Portd.6                                     ' SDA PB6 IC  Switched 0utput for ramp

  Swa Alias Pinb.7                                          ' D7  PB7 SCK Switch flank A
  Swb Alias Pinb.6                                          ' D6  PB6 MIS Switch flank B

  Red Alias Portb.5                                         ' D5  PB5 MOS Red charge control LED
  Green Alias Portb.4                                       ' D4  PB4     Green charge control LED

  Lamp_1d Alias Portb.2                                     ' D2  PB2     Lamp 1 dimmed
  Lamp_1 Alias Portb.3                                      ' D3  PB3 OC1 Lamp 1
  Lamp_2 Alias Portd.4                                      ' IO  PD4 T0  Lamp 2
  Lamp_3 Alias Portd.1                                      ' SO  PD1 TXD Lamp 3
  Lamp_4 Alias Portd.5                                      ' SCL PD5 T1  Lamp 4
  Lamp_5 Alias Portd.0                                      ' SI  PD0 RXD Lamp 5
  Lamp_6 Alias Portd.2                                      ' D8  PD2 IN0 Lamp 6
  Lamp_7 Alias Portd.3                                      ' D9  PD3 IN1 Lamp 7

 ' ****************************************************************************************

  Dim Result As Byte                                        ' Result And Intermediate Data
  Dim Tflag As Bit                                          ' Conversion flag

  Dim Charge As Byte                                        ' Battery charging value

  Dim First As Bit                                          ' Flank detection help
  Dim Trig As Bit                                           ' Flank detection recognized
  Dim Both As Bit                                           ' Positive brightness direction

  Dim Bright As Byte                                        ' Selected brightness
  Dim Lamps As Word                                         ' Number of burning lamps
  Dim Dimmer As Byte                                        ' A counter to dimm first lamp
  Dim Dimmerbit As Bit                                      ' First lamp current on

  Dim Service As Byte                                       ' Help counter for service mode

  Dim H As Bit                                              ' BASCOM Languague restriction patch

' ****************************************************************************************
'
'   PROGRAM BEGIN
'
' ****************************************************************************************
' Low Cost A/D Converter based on Atmel AVR 400
'
' To minimize the usage of external components, A/D converter uses the charging of a
' capacitor 22nF (controlled by portD.6)through a 12k..32k resistor while converting. By
' constricting the voltage range of the converter to 2/5 Vdd, the exponential curve is a
' good approximation of a straight line. This makes it possible to simply measure the time
' it takes before the voltage across the capacitor equals the voltage which is to be
' converted via use of the analog comparator. It will give an interrupt when the voltage
' across the capacitor rises above the measurement voltage.
' To ensure correct timing the time constant of the RC-network must satisfie
' 512*(1/f)=-R*C*ln(1-2/5). For the A/D converter to operate properly, the capacitor must
' be completly discharged between each conversion. This is done by allowing the
' discharging to take a minimum of 200us.
'                       The A/D conversion steps->
' 1. Call AD_convert 2. Wait for conversion complete 3. Read data from result
' ****************************************************************************************


Begin:
' ****************************************************************************************
' This part initializes the A/D converter. It sets the timer and the analog comparator.
' The analog comparator interrupt is being initiated by a rising edge On Ac0. To Enable
' the A / D Converter The Global Interurrupt Flag Must Be Set. Conversion flag is cleared.
' ****************************************************************************************

  Bright = 0                                                ' Lamps off a power on
  Service = 0                                               ' Service mode off
  First = 1                                                 ' To allow the first switch movement

  On Timer0 Ana_comp                                        ' Timer0 overflow handle
  On Aci Ana_comp                                           ' Analog comparator handle

  Config Timer0 = Timer , Prescale = 1                      ' Timer0 for analog messurement
  Acsr = &H0B                                               ' Initiate comparator and enable
'                                                     Comparator Interrupt
  Enable Timer0                                             ' Enable timer interrupt
  Enable Ovf0                                               ' Enable timer overflow
  Enable Aci                                                ' Enable Analog comperator interrupt

  Enable Interrupts                                         ' Enable global interrupt
  Set Vswitch                                               ' Set converter charge/discharge as output


' ****************************************************************************************
'
'   PROGRAM CYCLE - Cyclic execution starts here
'
' ****************************************************************************************

Again:
  Waitms 2                                                  ' Give time to discharge capacitor

' ****************************************************************************************
' Start one A/D conversion
' This routine starts the conversion. It loads the offset value into the timer0 and starts
' the timer. It also starts the charging of the capacitor.
' ****************************************************************************************

  Tcnt0 = Preset                                            ' Clear counter and load Offset Value
  Reset Tflag                                               ' Clear conversion complete flag (t)
  Tccr0 = &H02                                              ' Start timer0 with prescaling f/8
  Set Vswitch                                               ' Start charging of capacitor

' ****************************************************************************************
' Brightness switch detection
' Makes a flank detection and increases or decreases a value
' ****************************************************************************************

  Trig = Swa And First                                      ' Set on the first rise of pin SWA

  If Trig = 1 Then                                          ' Do it on first flank occure
     Reset First                                            ' Reset the flank detection
     Both = Swa And Swb                                     ' Swa and not Swb = decrement

     If Both = 1 Then                                       ' Increment condition detected

        Shift Bright , Left                                 ' Use one more lamp
        Incr Bright                                         ' and the smallest too
        Service = 0                                         ' Always reset service modus
     Else
        Shift Bright , Right                                ' Swith off highest lamp
        Incr Service                                        ' Go to Service modus at 255
     End If
  End If

  If Swa = 0 Then                                           ' Restart flank detection
     Set First                                              ' Prepares the flank detection
  End If

' ****************************************************************************************
' Dimmer for first lamp
' Uses a counter to dimm the first lamp via FET
' ****************************************************************************************

  If Dimmer > Dimmed_on Then                                ' Dimmer counter restriction
     Dimmer = 0
  End If

  Incr Dimmer                                               ' Dimmer from 1.. Dimmer_on

  If Dimmer <= Dimmed_off Then                              ' 1..Dimmer off
     Reset Dimmerbit                                        ' Lamp off
  Else
     Set Dimmerbit                                          ' Lamp on
  End If

' ****************************************************************************************
' Service modus to control charging messurement
' Decreases brightness value long time finaly starts the service modus. The results of the
' messurement will be reflected to the lamp outputs. To get good results make sure to do
' this tests with small lamps, to reduce voltage messurement errors by heavy load.
' ****************************************************************************************

  If Service >= 40 Then                                     ' Needs longer turning -> switch off
     Waitms 500                                             ' To avoid to fast mesures
     Lamps = Result                                         ' Shows voltage as value by byte
  Else
     Lamps = Bright                                         ' Normal usage of outputs
  End If

' ****************************************************************************************
' Brightness to lamps mixing
' Mixing / adaption of usd HW and program
' ****************************************************************************************

  H = Lamps.0 And Dimmerbit                                 ' Necessary BASCOM Languague patch
  Lamp_1d = H                                               'H                                               ' D2  PB2     Lamp 1 dimmed
  Lamp_1 = Lamps.1                                          ' D3  PB3 OC1 Lamp 1
  Lamp_2 = Lamps.2                                          ' IO  PD4 T0  Lamp 2
  Lamp_3 = Lamps.3                                          ' SO  PD1 TXD Lamp 3
  Lamp_4 = Lamps.4                                          ' SCL PD5 T1  Lamp 4
  Lamp_5 = Lamps.5                                          ' SI  PD0 RXD Lamp 5
  Lamp_6 = Lamps.6                                          ' D8  PD2 IN0 Lamp 6
  Lamp_7 = Lamps.7                                          ' D9  PD3 IN1 Lamp 7


' ****************************************************************************************
' Battery status indicator
' A dual LED red/green is used to indicate the battery status. Full is pure green and pure
' red is absolutely empty. Any status inbetween is indicated according following behaviour.
' As more green as higher the battery is charged. Missing charge is represented by orange.
' As soon as not full always a constant red pulse shows the beginning of a "controll cycle"
' ****************************************************************************************

  Decr Charge                                               ' Dual LED as battery indicator

  If Charge <= Empty Then                                   ' Restart battery status scan
     Charge = Full                                          ' Restart with over voltage
  End If

  If Charge >= Result Then
     If Result > Empty Then
        Set Red                                             ' Show used capacity as ORANGE
        Set Green
     Else
        Set Red
        Reset Green                                         ' Show undervoltage as pure RED
     End If
  Else
     If Charge >= Deep Then
        Reset Red
        Set Green                                           ' Show charged capacity as GREEN
     Else
        If Result < Full Then                               ' Allows blinking of meddium charge
           Reset Red
           Reset Green
        End If
     End If
  End If

Goto Again

End


' ****************************************************************************************
' Ana_comp - Analog comparator interrupt routine
' This routine is executed when one of two events occur:
'     1. At Timer/counter0 overflow interrupt  or at   2. Analog Comparator interrupt
' Both events signals the end of a conversion. Timer overflow if the signal is out of
' range and analog comparator if it is in range. The offset will be corrected, and the
' Tflag will be set. Due to the cycles needed for interruption handling, it is necessary
' to subtract 1 more than was added previously.
' ****************************************************************************************

Ana_comp:
  Result = Timer0                                           ' Load timer value
  Tccr0 = 0                                                 ' Stop timer0
  Result = Result - Preset                                  ' Rescale A/D output
  Result = Result - Offset                                  ' and the results ofthe last conversion
'                                                     are represented in Result.
  Reset Vswitch                                             ' Start discharge
  Set Tflag                                                 ' Set Conversion Complete Flag
Return                                                      ' Return from interrupt




